#include <utility.h>
#include <visa.h>
#include <ansi_c.h>
#include <cvirte.h>     
#include "RSSMU_WLAN_Example.h"
#include "rssmu.h"

/* Error check macro for driver functions */

#define CHECKERR( fCal ) \
    { \
        error = (fCal);\
        rssmu_error_message (io, error, error_message);\
        if (error < VI_SUCCESS) {\
            sprintf (error_buffer, "Primary Error: 0x%08X, %s\n", error, error_message);\
            MessagePopup ("Error", error_buffer); \
            rssmu_error_query (io, &error, error_message);\
            sprintf (error_buffer, "Secondary Error: 0x%08X, %s\n", error, error_message);\
            SetWaitCursor (0); \
            MessagePopup ("Error", error_buffer); \
            rssmu_close(io); \
            SetCtrlAttribute (panelHandle, PANEL_APPLY, ATTR_DIMMED, 0);\
            SetCtrlAttribute (panelHandle, PANEL_Exit, ATTR_DIMMED, 0);\
            break; \
        } \
    }
    
/* Globals */

ViInt32 panelHandle;
ViSession io;
ViStatus    error;
ViChar      error_message[256];
ViChar      error_buffer[256];

/* Local Functions */
int initialize (void);

int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)
        return -1;  /* out of memory */
    if ((panelHandle = LoadPanel (0, "RSSMU_WLAN_Example.uir", PANEL)) < 0)
        return -1;
    DisplayPanel (panelHandle);
    RunUserInterface ();
    DiscardPanel (panelHandle);
    return 0;
}

int CVICALLBACK exitApp (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
        {
        case EVENT_COMMIT:
            QuitUserInterface (0);
            break;
        }
    return 0;
}

int CVICALLBACK apllySettings (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    ViInt32     layer, data_source, bitrate, preamble, symbols, sync, seq_len;
    ViBoolean   state, wlan_state;
    ViReal64    rf_level, rf_freq, symbol_time;
    
    switch (event)
        {
        case EVENT_COMMIT:
             SetCtrlAttribute (panelHandle, PANEL_APPLY, ATTR_DIMMED, 1);
             SetCtrlAttribute (panelHandle, PANEL_Exit, ATTR_DIMMED, 1);
             
             ProcessDrawEvents ();
             
             if (initialize() < VI_SUCCESS) break;

             SetWaitCursor (1);

             GetCtrlVal (panelHandle, PANEL_WLAN_STATE, &wlan_state);
             GetCtrlVal (panelHandle, PANEL_RF_STATE, &state);
             
             //Clear Status
             CHECKERR (rssmu_ClearStatus(io));
             
             CHECKERR (rssmu_SetWLANState (io, RSSMU_PATH_A, VI_FALSE));
             


             GetCtrlVal (panelHandle, PANEL_PHYS_LAYER, &layer); 
             GetCtrlVal (panelHandle, PANEL_DATA_SOURCE, &data_source);
             GetCtrlVal (panelHandle, PANEL_SEQ_LENGTH, &seq_len);
             GetCtrlVal (panelHandle, PANEL_SYMBOL_TIME, &symbol_time);
             CHECKERR (rssmu_ConfigureWLANSettings (io, RSSMU_PATH_A,
                                                    RSSMU_WLAN_STANDARD_80211G,
                                                    layer,
                                                    RSSMU_WLAN_SIMULATION_FRAM,
                                                    RSSMU_WLAN_FRAME_DATA, seq_len, symbol_time));
             switch (layer)
             {
                case RSSMU_WLAN_PHYS_LAYER_OFDM:
                    GetCtrlVal (panelHandle, PANEL_OFDM_BITRATE, &bitrate); 
                    GetCtrlVal (panelHandle, PANEL_DATA_SYMBOLS, &symbols); 
                    CHECKERR (rssmu_ConfigureWLANPPDUforOFDM (io, RSSMU_PATH_A, bitrate, 1024,
                                                              data_source,
                                                              RSSMU_WLAN_SCRAMBLING_USER, symbols,
                                                              "0", VI_TRUE, "0", VI_FALSE, 100.0e-9));
                
                    break;
                default:
                    GetCtrlVal (panelHandle, PANEL_CCK_BITRATE, &bitrate); 
                    GetCtrlVal (panelHandle, PANEL_CCK_PREAMBLE, &preamble);
                    CHECKERR (rssmu_ConfigureWLANPPDUforCCKBPCC (io, RSSMU_PATH_A,
                                                       bitrate, 1024,
                                                       data_source,
                                                       RSSMU_WLAN_SCRAMBLING_ON,
                                                       preamble, VI_TRUE,
                                                       VI_TRUE));
                    
                    break;
             }

             GetCtrlVal (panelHandle, PANEL_RF_LEVEL, &rf_level);
             GetCtrlVal (panelHandle, PANEL_RF_FREQ, &rf_freq); 
             CHECKERR (rssmu_SetRFAmplitude (io, RSSMU_PATH_A, rf_level));
             CHECKERR (rssmu_SetRFFrequency (io, RSSMU_PATH_A, rf_freq));
             
             CHECKERR (rssmu_SetWLANState (io, RSSMU_PATH_A, wlan_state));     

             // send *OPC
             CHECKERR (rssmu_OperationComplete (io)); 
             
             // wait 
             do
             {  //  check if SMU is ready
                Delay(0.1);           
                CHECKERR (rssmu_getRegister (io, RSSMU_REGISTER_OPERATION, RSSMU_REGISTER_COND,&sync));
                
             } while ( (sync & 2) != 0);
             
             CHECKERR (rssmu_SetOutputState (io, RSSMU_PATH_A, state));

             rssmu_close(io);

             SetWaitCursor (0);
             SetCtrlAttribute (panelHandle, PANEL_APPLY, ATTR_DIMMED, 0);
             SetCtrlAttribute (panelHandle, PANEL_Exit, ATTR_DIMMED, 0);
            break;
        }
    return 0;
}

void CVICALLBACK panelHelp (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    if (panel == panelHandle)
        MessagePopup ("Help", "This is a simple demo program using\n"
                              "the Rohde&Schwarz SMU Vector Signal Generator\n"
                              "instrument driver provided in the package.\n");

    return;
}

/*===========================================================================*/ 
/* Local Functions Definitions                                               */
/*===========================================================================*/

/*===========================================================================*/
/* Function: Initialize                                                      */
/* Purpose:  This function initializes the instrument, performs ID Query     */
/*           and Reset                                                       */
/*===========================================================================*/
int initialize (void)
{
    ViChar      instr_desc[256];
    ViBoolean   id;
    ViBoolean   rst;
    

    SetWaitCursor(0);
    GetCtrlVal (panelHandle, PANEL_INSTR_DESC, instr_desc);
    GetCtrlVal (panelHandle, PANEL_IDQUERY, &id);
    GetCtrlVal (panelHandle, PANEL_RESET, &rst);
    SetWaitCursor(1);
    if ((error = rssmu_init (instr_desc, id, rst, &io)) < 0 )
    {
        rssmu_error_message(VI_NULL, error, error_message);
        sprintf (error_buffer, "Initialization Error:\n\n%s\n\nCheck your "
                "connections and make sure you have the right instrument descriptor.",
                 error_message);
        MessagePopup ("ERROR!", error_buffer);
        SetWaitCursor(0);
        return(-1);   
    }

    viSetAttribute (io, VI_ATTR_TMO_VALUE, 10000);
    SetWaitCursor(0);
    return 0;
}
